
/* htmlio.cc */

#include "htmlio.h"

#include "str.h"

OPEN_NAMESPACE

/********************************* HTML STYLE **********************************/

void THtmlOutput::Style ()
{
   PutLn (".namespace { color: Gold; }");
   PutLn (".type { color: Chocolate; }");
   PutLn (".class { color: Blue; }");
   PutLn (".enum { color: DarkOrange; }");
   PutLn (".template { color: DarkViolet; }");
   PutLn (".variable { color: SaddleBrown; }");
   PutLn (".function { color: Green; }");

   PutLn (".class_group { color: #3399ff; }");
   PutLn (".function_group { color: Lime; }");
   PutLn (".overloaded_function { color: Olive; }");
   PutLn (".overloaded_function_group { color: Chartreuse; }");

   PutLn (".template_instance { color: #780078; }");
   PutLn (".template_specialization { color: BlueViolet; }");

   PutLn (".template_param { color: Magenta; }");
   PutLn (".nested_template_param { color: Coral; }");

   PutLn (".function_template { color: Turquoise; }");
   PutLn (".function_template_instance { color: LightSeaGreen; }");
   PutLn (".function_template_specialization { color: Aquamarine; }");

   PutLn (".directive { color: LimeGreen; }");
   PutLn (".keyword { color: FireBrick; }");
   PutLn (".ident { color: Navy; }");
   PutLn (".number { color: RoyalBlue; }");
   PutLn (".float { color: Purple; }");
   PutLn (".string { color: Blue; }");
   PutLn (".character { color: CornflowerBlue; }");
   PutLn (".separator { color: Navy; }");

   PutLn (".rule { color: OrangeRed; }");
   PutLn (".terminal { color: Blue; }");
   PutLn (".nonterminal { color: SaddleBrown; }");
   PutLn (".grammar { color: CornflowerBlue; }");

   PutLn (".comment { color: Grey; /* font-style:italic; */ }");
   PutLn (".property { color: CornFlowerBlue; }");

   PutLn (".line_number { color: Grey; }");
   PutLn (".address { color: #3399ff; }");
   PutLn (".jump { color: CornflowerBlue; }");

   PutLn (".key { color: Blue; }");
   PutLn (".value { color: Orange; }");

   PutLn (".other { color: Orange; }");
   PutLn (".unknown { color: Red; }");

   PutLn (".info { color: Blue; }");
   PutLn (".warning { color: Orange; }");
   PutLn (".error { color: Red; }");

   PutLn ("h1 {");
   IncIndent ();
   PutLn ("margin: 0.5em auto 0.5em auto;");
   PutLn ("padding: 15px 10px 15px 55px;");
   PutLn ("color: Blue;");
   PutLn ("background: #ffff70;");
   PutLn ("border: 1px solid #c0c0e0;");
   DecIndent ();
   PutLn ("}");

   PutLn ("h2 {");
   IncIndent ();
   PutLn ("margin: 0.5em auto 0.5em auto;");
   PutLn ("padding: 10px 10px 10px 55px;");
   PutLn ("color: Blue;");
   PutLn ("background: #ffff90;");
   PutLn ("border: 1px solid #c0c0e0;");
   DecIndent ();
   PutLn ("}");

   PutLn ("h3 {");
   IncIndent ();
   PutLn ("margin: 0.5em auto 0.5em auto;");
   PutLn ("padding: 5px 5px 5px 55px;");
   PutLn ("color: Blue;");
   PutLn ("background: #ffffb0;");
   PutLn ("border: 1px solid #c0c0e0;");
   DecIndent ();
   PutLn ("}");

}

/********************************* HTML HELPER *********************************/

THtmlOutput::THtmlOutput () :
      fUseHtml (true)
{
}

THtmlOutput::~THtmlOutput ()
{
}

inline string FceAttr (const string name, const string value)
{
   return name + "=" + quote2 + QuoteStrContent (value) + quote2;
}

void THtmlOutput::Attr (const string name, const string value)
// name = "value"
{
   if (fUseHtml)
   {
      Put (name);
      PutChr ('=');
      PutChr (quote2);
      Put (QuoteStrContent (value));
      PutChr (quote2);
   }
}

void THtmlOutput::Href (const string url, const string style, const string text)
// <a href="url" class="style"> text </a>
{
   if (fUseHtml)
   {
      Put ("<a ");
      Attr ("href", url);

      if (style != "")
      {
         Put (" ");
         Attr ("class", style);
      }

      Put (">");

      Put (text);
      Put ("</a>");
   }
}

void THtmlOutput::AName (const string name, const string style, const string text)
// <a name="name"> text </a>
{
   if (fUseHtml)
   {
      Put ("<a ");
      Attr ("name", name);

      if (style != "")
      {
         Put (" ");
         Attr ("class", style);
      }

      if (text == "")
      {
         Put ("/>");
      }
      else
      {
         Put (">");
         Put (text);
         Put ("</a>");
      }
   }
   else
   {
      Put (text);
   }
}

void THtmlOutput::StyleBegin (const string style)
{
   if (fUseHtml)
   {
      Put ("<span ");
      Attr ("class", style);
      Put (">");
   }
}

void THtmlOutput::StyleEnd ()
{
   if (fUseHtml)
      Put ("</span>");
}

void THtmlOutput::StyledText (const string text, const string style)
{
   if (fUseHtml)
   {
      Put ("<span ");
      Attr ("class", style);
      Put (">");
      Put (text);
      Put ("</span>");
   }
   else
   {
      Put (text);
   }
}

void THtmlOutput::PreBegin ()
{
   if (fUseHtml)
      PutLn ("<pre>");
}

void THtmlOutput::PreEnd ()
{
   if (fUseHtml)
      PutLn ("</pre>");
}

void THtmlOutput::PutHtml (const string s)
{
   if (fUseHtml)
      Put (s);
}

void THtmlOutput::PutHtmlLn (const string s)
{
   if (fUseHtml)
      PutLn (s);
}

void THtmlOutput::PutNorm (const string s)
{
   if (! fUseHtml)
      Put (s);
}

void THtmlOutput::PutNormLn (const string s)
{
   if (! fUseHtml)
      PutLn (s);
}

string THtmlOutput::HtmlEscape (const string s)
{
   int len = s.length ();

   bool ok = true;

   for (int i = 0; i < len; i++)
   {
      char c = s[i];

      if (c == '<' || c == '>' || c == '&')
         ok = false;
   }

   if (ok)
   {
      return s;
   }
   else
   {
      string result = "";

      for (int i = 0; i < len; i++)
      {
         char c = s[i];

         if (c == '<')
            result += "&lt;";
         else if (c == '>')
            result += "&gt;";
         else if (c == '&')
            result += "&amp;";
         else
            result += c;
      }

      return result;
   }
}

void THtmlOutput::PutHtmlEscape (const string s)
// Replace < > & characters
{
   if (fUseHtml)
   {
      int len = s.length ();

      bool ok = true;

      for (int i = 0; i < len; i++)
      {
         char c = s[i];

         if (c == '<' || c == '>' || c == '&')
            ok = false;
      }

      if (ok)
      {
         Put (s);
      }
      else
      {
         for (int i = 0; i < len; i++)
         {
            char c = s[i];

            if (c == '<')
               Put ("&lt;");
            else if (c == '>')
               Put ("&gt;");
            else if (c == '&')
               Put ("&amp;");
            else
               PutChr (c);
         }
      }
   }
   else
   {
      Put (s);
   }
}

void THtmlOutput::Head (const string title)
{
   if (fUseHtml)
   {
      string charset = "ISO-8859-1";

      PutLn ("<?xml " + FceAttr ("version", "1.0") + "?>");

      PutLn ("<!DOCTYPE html PUBLIC " +
             QuoteStr ("-//W3C//DTD XHTML 1.0 Transitional//EN") + " " +
             QuoteStr ("http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd") + ">");

      PutLn ("<html " + FceAttr ("xmlns", "http://www.w3.org/1999/xhtml") + " " +
             FceAttr ("xml:lang", "en") + " " +
             FceAttr ("lang", "en") + ">");

      PutLn ("<head>");

      PutLn ("<meta " + FceAttr ("http-equiv", "Content-Type") + " " +
             FceAttr ("content", "text/html; charset=" + charset) + " />");

      PutLn ("<title>" + title + "</title>");

      if (fStyleFileName != "")
      {
         PutLn ("<link " + FceAttr ("rel", "stylesheet") + " " +
                FceAttr ("type", "text/css") + " " +
                FceAttr ("href", fStyleFileName) + " />");
      }
      else
      {
         PutLn ("<style " + FceAttr ("type", "text/css") + ">");
         IncIndent ();
         Style ();
         DecIndent ();
         PutLn ("</style>");
      }

      PutLn ("</head>");
      PutLn ("<body>");
   }
}


void THtmlOutput::Tail ()
{
   if (fUseHtml)
   {
      PutLn ("</body>");
      PutLn ("</html>");
   }
}

CLOSE_NAMESPACE
